import {
  ApplicationContext,
  Catch,
  IMidwayContainer,
  Inject,
} from '@midwayjs/core';
import { Context } from '@midwayjs/koa';
import { LoginError } from '../error/login.error';
import { ServiceError } from '../error/service.error';
import { MidwayI18nService } from '@midwayjs/i18n';
import { toLower } from 'lodash';
interface CustomError extends Error {
  group: string;
}

@Catch([LoginError, ServiceError])
export class CustomErrorFilter {
  @Inject()
  ctx: Context;
  @ApplicationContext()
  applicationContext: IMidwayContainer;
  async catch(err: CustomError, ctx: Context) {
    // 所有的未分类错误会到这里
    const i18n = await this.applicationContext.getAsync(MidwayI18nService);
    const errmsg = i18n.translate(err.message, {
      locale: this.ctx.locale,
      // 需要全小写
      group: toLower(err.group),
    });
    const result: any = {
      errcode: err.message,
      errmsg,
    };
    if (process.env.NODE_ENV !== 'production') {
      result.details = err.stack;
    }
    return result;
  }
}
